with Ada.Text_IO;

with Names;
with AocsMgr;
with ModeMgr;
with UnitMgr;

package body FdirMgr is

   use Names;


   Units : Units_T renames UnitMgr.Units;

   Request_Mode : Target_Mode_T := None;

   procedure Handle_Attitude_Error is
   begin
      Ada.Text_IO.Put_Line ("FDIR: Handle_Attitude_Error m="
         & ModeMgr.Mode'Img
         & " t=" & ModeMgr.Trans.Target'Img);

-- FdirMgr.Main execution statement   15
      case ModeMgr.Mode is

         when Science =>

-- FdirMgr.Main execution statement   16
            Request_Mode := Preparation;

         when Preparation =>

-- FdirMgr.Main execution statement   17
            Request_Mode := Nominal;

         when Nominal =>

-- FdirMgr.Main execution statement   18
            Request_Mode := Safe;

         when others =>

            -- Emergency

            Ada.Text_IO.Put_Line ("FDIR: Request_CDMU_Reboot");
-- FdirMgr.Main execution statement   19
            Request_Mode := Off;

      end case;

   end Handle_Attitude_Error;


   procedure Handle_Mode_Transition_Error is
      -- Invariant: ModeMgr.Trans.Target = None
   begin
      Ada.Text_IO.Put_Line ("FDIR: Handle_Mode_Transition_Error"
         & " m=" & ModeMgr.Mode'Img
         & " t=" & ModeMgr.Trans.Target'Img);

-- FdirMgr.Main execution statement    3
      UnitMgr.Abort_All_Reconfigurations;

-- FdirMgr.Main execution statement   10
      if ModeMgr.Mode = Off
         or ModeMgr.Mode = Standby
         or ModeMgr.Trans.Target = Safe
      then
         -- Emergency

         Ada.Text_IO.Put_Line ("FDIR: Request_CDMU_Reboot");
-- FdirMgr.Main execution statement   11
         Request_Mode := Off;

      else

-- FdirMgr.Main execution statement   12
         Request_Mode := Safe;

      end if;

   end Handle_Mode_Transition_Error;


   procedure Handle_Unit_Error (UnId : Unit_ID_T)
   is
      -- Invariant: ModeMgr.Mode > Standby
      -- and Units(UnId).Br(Units(UnId).CuBrId).Status /= Free
      Ret_Value_from_Call : Boolean;
   begin
      Ada.Text_IO.Put_Line ("FDIR: Handle_Unit_Error"
         & " u=" & UnId'Img
         & " br=" & Units(UnId).CuBrId'Img
         & " err=" & Units(UnId).Br(Units(UnId).CuBrId).Error'Img
         & " c=" & Units(UnId).Br(Units(UnId).CuBrId).Error_Counter'Img);

-- FdirMgr.Main execution statement   24
      if UnId = PLI
         and ModeMgr.Mode = Science
         and Units(UnId).Br(Units(UnId).CuBrId).Error = Loss_Of_Accuracy
         and Units(UnId).Br(Units(UnId).CuBrId).Error_Counter < 5
      then

-- FdirMgr.Main execution statement   25
         Request_Mode := Preparation;

      else
          
-- FdirMgr.Main execution statement   26
         Ret_Value_from_Call := UnitMgr.Redundancy_Available (UnId);
-- FdirMgr.Main execution statement   29
         if Ret_Value_from_Call then

-- FdirMgr.Main execution statement   30
            UnitMgr.Start_Switch_To_Redundant (UnId);

-- FdirMgr.Main execution statement   37
         elsif UnId = PLI then
            -- PLI not used in Nominal Mode

-- FdirMgr.Main execution statement   38
            Request_Mode := Nominal;

-- FdirMgr.Main execution statement   39
         elsif UnitMgr.First_State (Safe, UnId) = Off then
            -- Unit not used in SafeMode

-- FdirMgr.Main execution statement   40
            Request_Mode := Safe;

         else
            -- Emergency: unit needed in SafeMode, and
            -- both unit branches are failed

            Ada.Text_IO.Put_Line ("FDIR: Request_CDMU_Reboot");
-- FdirMgr.Main execution statement   41
            Request_Mode := Off;

         end if;
      end if;

   end Handle_Unit_Error;


   procedure Main is
   begin

-- FdirMgr.Main execution statement    1
      if ModeMgr.Trans.Error /= None then

-- FdirMgr.Main execution statement    2
         Handle_Mode_Transition_Error;

-- FdirMgr.Main execution statement   13
      elsif AocsMgr.Control_Law.Error /= None then

-- FdirMgr.Main execution statement   14
         Handle_Attitude_Error;

      else

-- FdirMgr.Main execution statement   20
         for UnId in Unit_ID_T loop
            Ada.Text_IO.Put_Line ("FDIR unit err="
               & Units(UnId).Br(Units(UnId).CuBrId).Error'Img);
-- FdirMgr.Main execution statement   21
            if Units(UnId).Br(Units(UnId).CuBrId).Error /= None
               and Units(UnId).Br(Units(UnId).CuBrId).Target_State /= Off
               -- Do not handle unit which is switched off
               and (
               Units(UnId).Br(Units(UnId).CuBrId).State /= Off
               or Units(UnId).Br(Units(UnId).CuBrId).Target_State /= None )
               -- Do not reconfigure unit which is Off
               -- and not currently switched on
            then
-- FdirMgr.Main execution statement   22
               Units(UnId).Br(Units(UnId).CuBrId).Error_Counter :=
                  Units(UnId).Br(Units(UnId).CuBrId).Error_Counter + 1;
-- FdirMgr.Main execution statement   23
               Handle_Unit_Error (UnId);
-- FdirMgr.Main execution statement   42
               if Request_Mode /= None then
-- FdirMgr.Main execution statement   43
                  ModeMgr.Set_FDIR_Request (Request_Mode);
-- FdirMgr.Main execution statement   57
                  Request_Mode := None;
               end if;
            end if;

-- FdirMgr.Main execution statement 1020
         end loop;

      end if;

-- FdirMgr.Main execution statement   58
      if Request_Mode /= None then
-- FdirMgr.Main execution statement   59
         ModeMgr.Set_FDIR_Request (Request_Mode);
-- FdirMgr.Main execution statement   73
         Request_Mode := None;
      end if;

-- FdirMgr.Main execution statement   74
      for UnId in Unit_ID_T loop
-- FdirMgr.Main execution statement   75
         Units(UnId).Br(Units(UnId).CuBrId).Error := None;
-- FdirMgr.Main execution statement 1074
      end loop;

-- FdirMgr.Main execution statement   76
      ModeMgr.Trans.Error := None;

-- FdirMgr.Main execution statement   77
      AocsMgr.Control_Law.Error := None;

   end Main;


end FdirMgr;
