with Ada.Characters.Handling;
with Ada.Text_IO;

with Names;
with UnitMgr.Branch_Control;

package body UnitMgr is

   use Names;

   procedure Execute_Reconf_Step (UnId : Unit_ID_T);
   -- Body declared later in this file

   procedure Main is
   begin

-- UnitMgr.Main execution statement    1
      for UnId in Unit_ID_T loop

         -- Continue possible unit level procedures:

-- UnitMgr.Main execution statement    2
         if Units(UnId).Reconf_Ongoing then

-- UnitMgr.Main execution statement    3
            Execute_Reconf_Step (UnId);
            -- Every unit has its own reconfiguration procedure taking
            -- care of the step condition evaluation and possible
            -- update of Units(UnId).Step. When the reconfiguration is over,
            -- branch information is updated accordingly,
            -- and Units(UnId).Reconf_Ongoing is set
            -- to False to indicate that procedure is over.

         end if;

         -- Continue possible branch level procedures:

-- UnitMgr.Main execution statement   32
         for BrId in Branch_ID_T loop
-- UnitMgr.Main execution statement   33
            Branch_Control.Possible_State_Transition_Step (UnId, BrId);
-- UnitMgr.Main execution statement 1032
         end loop;
         Ada.Text_IO.Put ("UNITMGR u=" & UnId'Img
            & " re=" & Units(UnId).Reconf_Ongoing'Img);
         for BrId in Branch_ID_T loop
            Ada.Text_IO.Put (" "
               & Ada.Characters.Handling.To_Lower(BrId'Img)
               & "st=" & Units(UnId).Br(BrId).State'Img & " "
               & Ada.Characters.Handling.To_Lower(BrId'Img)
               & "tr=" & Units(UnId).Br(BrId).Target_State'Img & " "
               & Ada.Characters.Handling.To_Lower(BrId'Img)
               & "ss=" & Units(UnId).Br(BrId).Status'Img);
         end loop;
         Ada.Text_IO.New_Line;
-- UnitMgr.Main execution statement 1001
      end loop;

   end Main;


   function Redundancy_Available (UnId : Unit_ID_T) return Boolean
   is
      Ret_Value : Boolean;
   begin

-- FdirMgr.Main execution statement   27
      Ret_Value := (
         not Units(UnId).Reconf_Ongoing
         and Units(UnId).CuBrId = A);
-- FdirMgr.Main execution statement   28
      return Ret_Value;

   end Redundancy_Available;


   procedure Init_Reconf_Proc (UnId : Unit_ID_T) is
   begin

-- FdirMgr.Main execution statement    6
-- ModeMgr.Main execution statement   59
-- ModeMgr.Main execution statement   98
-- ModeMgr.Main execution statement  140
-- ModeMgr.Main execution statement  162
-- ModeMgr.Main execution statement  243
-- ModeMgr.Main execution statement  265
-- ModeMgr.Main execution statement  339
-- ModeMgr.Main execution statement  381
-- ModeMgr.Main execution statement  453
-- ModeMgr.Main execution statement  474
-- ModeMgr.Main execution statement  528
-- ModeMgr.Main execution statement  587
      Units(UnId).Reconf_Ongoing := False;

-- FdirMgr.Main execution statement    7
-- ModeMgr.Main execution statement   60
-- ModeMgr.Main execution statement   99
-- ModeMgr.Main execution statement  141
-- ModeMgr.Main execution statement  163
-- ModeMgr.Main execution statement  244
-- ModeMgr.Main execution statement  266
-- ModeMgr.Main execution statement  340
-- ModeMgr.Main execution statement  382
-- ModeMgr.Main execution statement  454
-- ModeMgr.Main execution statement  475
-- ModeMgr.Main execution statement  529
-- ModeMgr.Main execution statement  588
      Units(UnId).Orig_Status   := Free;
-- FdirMgr.Main execution statement    8
-- ModeMgr.Main execution statement   61
-- ModeMgr.Main execution statement  100
-- ModeMgr.Main execution statement  142
-- ModeMgr.Main execution statement  164
-- ModeMgr.Main execution statement  245
-- ModeMgr.Main execution statement  267
-- ModeMgr.Main execution statement  341
-- ModeMgr.Main execution statement  383
-- ModeMgr.Main execution statement  455
-- ModeMgr.Main execution statement  476
-- ModeMgr.Main execution statement  530
-- ModeMgr.Main execution statement  589
      Units(UnId).Target_State  := Off;
-- FdirMgr.Main execution statement    9
-- ModeMgr.Main execution statement   62
-- ModeMgr.Main execution statement  101
-- ModeMgr.Main execution statement  143
-- ModeMgr.Main execution statement  165
-- ModeMgr.Main execution statement  246
-- ModeMgr.Main execution statement  268
-- ModeMgr.Main execution statement  342
-- ModeMgr.Main execution statement  384
-- ModeMgr.Main execution statement  456
-- ModeMgr.Main execution statement  477
-- ModeMgr.Main execution statement  531
-- ModeMgr.Main execution statement  590
      Units(UnId).Step          := 0;

   end Init_Reconf_Proc;


   procedure Abort_All_Reconfigurations is
   begin

-- FdirMgr.Main execution statement    4
      for UnId in Unit_ID_T loop

-- FdirMgr.Main execution statement    5
         Init_Reconf_Proc (UnId);

-- FdirMgr.Main execution statement 1004
      end loop;

   end Abort_All_Reconfigurations;


   function No_Reconfiguration return Boolean is
      Ret_Value : Boolean;
   begin

-- ModeMgr.Main execution statement  207
-- ModeMgr.Main execution statement  310
-- ModeMgr.Main execution statement  355
-- ModeMgr.Main execution statement  427
      for UnId in Unit_ID_T loop

-- ModeMgr.Main execution statement  208
-- ModeMgr.Main execution statement  311
-- ModeMgr.Main execution statement  356
-- ModeMgr.Main execution statement  428
         if Units(UnId).Reconf_Ongoing then

-- ModeMgr.Main execution statement  209
-- ModeMgr.Main execution statement  312
-- ModeMgr.Main execution statement  357
-- ModeMgr.Main execution statement  429
            Ret_Value := False;
-- ModeMgr.Main execution statement  210
-- ModeMgr.Main execution statement  313
-- ModeMgr.Main execution statement  358
-- ModeMgr.Main execution statement  430
            return Ret_Value;

         end if;

-- ModeMgr.Main execution statement 1207
-- ModeMgr.Main execution statement 1310
-- ModeMgr.Main execution statement 1355
-- ModeMgr.Main execution statement 1427
      end loop;

-- ModeMgr.Main execution statement  211
-- ModeMgr.Main execution statement  314
-- ModeMgr.Main execution statement  359
-- ModeMgr.Main execution statement  431
      Ret_Value := True;
-- ModeMgr.Main execution statement  212
-- ModeMgr.Main execution statement  315
-- ModeMgr.Main execution statement  360
-- ModeMgr.Main execution statement  432
      return Ret_Value;

   end No_Reconfiguration;


   procedure Start_Switch_To_Redundant (UnId : Unit_ID_T)
   is
      -- Invariant: Units(UnId).Reconf_Ongoing = False
   begin
      Ada.Text_IO.Put_Line ("RECONF u="&UnId'Img
         & " " & Units(UnId).CuBrId'Img);

-- FdirMgr.Main execution statement   31
      if Units(UnId).Br(Units(UnId).CuBrId).Target_State /= Off
         -- Do not reconfigure unit which is switched off
         and (Units(UnId).Br(Units(UnId).CuBrId).State /= Off
             or Units(UnId).Br(Units(UnId).CuBrId).Target_State /= None)
         -- nor unit which is off
      then

-- FdirMgr.Main execution statement   32
         Units(UnId).Orig_Status :=
           Units(UnId).Br(Units(UnId).CuBrId).Status;

-- FdirMgr.Main execution statement   33
         if Units(UnId).Br(Units(UnId).CuBrId).Target_State /= None then
-- FdirMgr.Main execution statement   34
            Units(UnId).Target_State :=
              Units(UnId).Br(Units(UnId).CuBrId).Target_State;
         else
-- FdirMgr.Main execution statement   35
            Units(UnId).Target_State :=
              Units(UnId).Br(Units(UnId).CuBrId).State;
         end if;

-- FdirMgr.Main execution statement   36
         Units(UnId).Reconf_Ongoing := True;

      end if;

   end Start_Switch_To_Redundant;


   procedure Internal_Start_Trans (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T;
      Target_State : Branch_State_T )
   is
   begin

-- ModeMgr.Main execution statement   68
-- ModeMgr.Main execution statement  107
-- ModeMgr.Main execution statement  149
-- ModeMgr.Main execution statement  171
-- ModeMgr.Main execution statement  252
-- ModeMgr.Main execution statement  274
-- ModeMgr.Main execution statement  348
-- ModeMgr.Main execution statement  390
-- ModeMgr.Main execution statement  462
-- ModeMgr.Main execution statement  483
-- ModeMgr.Main execution statement  537
-- ModeMgr.Main execution statement  596
-- UnitMgr.Main execution statement    9
-- UnitMgr.Main execution statement   21
      UnitMgr.Branch_Control.Init_Transition (UnId, BrId);

-- ModeMgr.Main execution statement   72
-- ModeMgr.Main execution statement  111
-- ModeMgr.Main execution statement  153
-- ModeMgr.Main execution statement  175
-- ModeMgr.Main execution statement  256
-- ModeMgr.Main execution statement  278
-- ModeMgr.Main execution statement  352
-- ModeMgr.Main execution statement  394
-- ModeMgr.Main execution statement  466
-- ModeMgr.Main execution statement  487
-- ModeMgr.Main execution statement  541
-- ModeMgr.Main execution statement  600
-- UnitMgr.Main execution statement   13
-- UnitMgr.Main execution statement   25
      Units(UnId).Br(BrId).Target_State := Target_State;

   end Internal_Start_Trans;


   procedure Start_Transition_Br (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T;
      Target_State : Branch_State_T )
   is
      -- Invariants:
      -- Current branch state acceptable for Target_State
      -- other than PLI, GPS: State /= Off implies Target_State = None
      -- GPS: State = Fine_Nav and Target = Coarse_Nav implies Error = LOA
      -- GPS: to be specified
      -- PLI: to be specified
   begin

-- ModeMgr.Main execution statement   54
-- ModeMgr.Main execution statement   93
-- ModeMgr.Main execution statement  135
-- ModeMgr.Main execution statement  157
-- ModeMgr.Main execution statement  238
-- ModeMgr.Main execution statement  260
-- ModeMgr.Main execution statement  334
-- ModeMgr.Main execution statement  376
-- ModeMgr.Main execution statement  448
-- ModeMgr.Main execution statement  469
-- ModeMgr.Main execution statement  523
-- ModeMgr.Main execution statement  582
      if Target_State = Off then

-- ModeMgr.Main execution statement   55
-- ModeMgr.Main execution statement   94
-- ModeMgr.Main execution statement  136
-- ModeMgr.Main execution statement  158
-- ModeMgr.Main execution statement  239
-- ModeMgr.Main execution statement  261
-- ModeMgr.Main execution statement  335
-- ModeMgr.Main execution statement  377
-- ModeMgr.Main execution statement  449
-- ModeMgr.Main execution statement  470
-- ModeMgr.Main execution statement  524
-- ModeMgr.Main execution statement  583
         Units(UnId).Br(BrId).Error := None;
-- ModeMgr.Main execution statement   56
-- ModeMgr.Main execution statement   95
-- ModeMgr.Main execution statement  137
-- ModeMgr.Main execution statement  159
-- ModeMgr.Main execution statement  240
-- ModeMgr.Main execution statement  262
-- ModeMgr.Main execution statement  336
-- ModeMgr.Main execution statement  378
-- ModeMgr.Main execution statement  450
-- ModeMgr.Main execution statement  471
-- ModeMgr.Main execution statement  525
-- ModeMgr.Main execution statement  584
         Units(UnId).Br(BrId).Error_Counter := 0;

-- ModeMgr.Main execution statement   57
-- ModeMgr.Main execution statement   96
-- ModeMgr.Main execution statement  138
-- ModeMgr.Main execution statement  160
-- ModeMgr.Main execution statement  241
-- ModeMgr.Main execution statement  263
-- ModeMgr.Main execution statement  337
-- ModeMgr.Main execution statement  379
-- ModeMgr.Main execution statement  451
-- ModeMgr.Main execution statement  472
-- ModeMgr.Main execution statement  526
-- ModeMgr.Main execution statement  585
         if Units (UnId).Reconf_Ongoing then
            -- Request comes outside of this package,
            -- abort currently running reconfiguration

-- ModeMgr.Main execution statement   58
-- ModeMgr.Main execution statement   97
-- ModeMgr.Main execution statement  139
-- ModeMgr.Main execution statement  161
-- ModeMgr.Main execution statement  242
-- ModeMgr.Main execution statement  264
-- ModeMgr.Main execution statement  338
-- ModeMgr.Main execution statement  380
-- ModeMgr.Main execution statement  452
-- ModeMgr.Main execution statement  473
-- ModeMgr.Main execution statement  527
-- ModeMgr.Main execution statement  586
            Init_Reconf_Proc (UnId);

         end if;

-- ModeMgr.Main execution statement   63
-- ModeMgr.Main execution statement  102
-- ModeMgr.Main execution statement  144
-- ModeMgr.Main execution statement  166
-- ModeMgr.Main execution statement  247
-- ModeMgr.Main execution statement  269
-- ModeMgr.Main execution statement  343
-- ModeMgr.Main execution statement  385
-- ModeMgr.Main execution statement  457
-- ModeMgr.Main execution statement  478
-- ModeMgr.Main execution statement  532
-- ModeMgr.Main execution statement  591
         Units(UnId).Br(BrId).Status := Free;

-- ModeMgr.Main execution statement   64
-- ModeMgr.Main execution statement  103
-- ModeMgr.Main execution statement  145
-- ModeMgr.Main execution statement  167
-- ModeMgr.Main execution statement  248
-- ModeMgr.Main execution statement  270
-- ModeMgr.Main execution statement  344
-- ModeMgr.Main execution statement  386
-- ModeMgr.Main execution statement  458
-- ModeMgr.Main execution statement  479
-- ModeMgr.Main execution statement  533
-- ModeMgr.Main execution statement  592
      elsif Units(UnId).Br(BrId).Status = Free then

-- ModeMgr.Main execution statement   65
-- ModeMgr.Main execution statement  104
-- ModeMgr.Main execution statement  146
-- ModeMgr.Main execution statement  168
-- ModeMgr.Main execution statement  249
-- ModeMgr.Main execution statement  271
-- ModeMgr.Main execution statement  345
-- ModeMgr.Main execution statement  387
-- ModeMgr.Main execution statement  459
-- ModeMgr.Main execution statement  480
-- ModeMgr.Main execution statement  534
-- ModeMgr.Main execution statement  593
         for BrIdAux in Branch_ID_T loop
-- ModeMgr.Main execution statement   66
-- ModeMgr.Main execution statement  105
-- ModeMgr.Main execution statement  147
-- ModeMgr.Main execution statement  169
-- ModeMgr.Main execution statement  250
-- ModeMgr.Main execution statement  272
-- ModeMgr.Main execution statement  346
-- ModeMgr.Main execution statement  388
-- ModeMgr.Main execution statement  460
-- ModeMgr.Main execution statement  481
-- ModeMgr.Main execution statement  535
-- ModeMgr.Main execution statement  594
            Units(UnId).Br(BrIdAux).Status := Reserved;
-- ModeMgr.Main execution statement 1065
-- ModeMgr.Main execution statement 1104
-- ModeMgr.Main execution statement 1146
-- ModeMgr.Main execution statement 1168
-- ModeMgr.Main execution statement 1249
-- ModeMgr.Main execution statement 1271
-- ModeMgr.Main execution statement 1345
-- ModeMgr.Main execution statement 1387
-- ModeMgr.Main execution statement 1459
-- ModeMgr.Main execution statement 1480
-- ModeMgr.Main execution statement 1534
-- ModeMgr.Main execution statement 1593
         end loop;
         -- We reserve both branches, since the request came from
         -- ModeMgr.

      end if;

-- ModeMgr.Main execution statement   67
-- ModeMgr.Main execution statement  106
-- ModeMgr.Main execution statement  148
-- ModeMgr.Main execution statement  170
-- ModeMgr.Main execution statement  251
-- ModeMgr.Main execution statement  273
-- ModeMgr.Main execution statement  347
-- ModeMgr.Main execution statement  389
-- ModeMgr.Main execution statement  461
-- ModeMgr.Main execution statement  482
-- ModeMgr.Main execution statement  536
-- ModeMgr.Main execution statement  595
      Internal_Start_Trans (UnId, BrId, Target_State);

   end Start_Transition_Br;


   procedure Start_Transition_U (
      UnId : Unit_ID_T;
      Target_State : Branch_State_T )
   is
   begin
-- ModeMgr.Main execution statement  375
-- ModeMgr.Main execution statement  447
-- ModeMgr.Main execution statement  468
-- ModeMgr.Main execution statement  522
      Start_Transition_Br (UnId, Units(UnId).CuBrId, Target_State);
   end Start_Transition_U;

   function Trans_Done (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T;
      State : Branch_State_T )
   return Boolean
   is
      Ret_Value : Boolean;
   begin

-- ModeMgr.Main execution statement  120
-- ModeMgr.Main execution statement  183
-- ModeMgr.Main execution statement  286
-- ModeMgr.Main execution statement  367
-- ModeMgr.Main execution statement  401
-- ModeMgr.Main execution statement  413
-- ModeMgr.Main execution statement  439
-- ModeMgr.Main execution statement  494
-- ModeMgr.Main execution statement  506
-- ModeMgr.Main execution statement  548
-- ModeMgr.Main execution statement  609
-- UnitMgr.Main execution statement   16
-- UnitMgr.Main execution statement   28
      Ret_Value := (
         Units(UnId).Br(BrId).State = State
         and Units(UnId).Br(BrId).Target_State = None );
-- ModeMgr.Main execution statement  121
-- ModeMgr.Main execution statement  184
-- ModeMgr.Main execution statement  287
-- ModeMgr.Main execution statement  368
-- ModeMgr.Main execution statement  402
-- ModeMgr.Main execution statement  414
-- ModeMgr.Main execution statement  440
-- ModeMgr.Main execution statement  495
-- ModeMgr.Main execution statement  507
-- ModeMgr.Main execution statement  549
-- ModeMgr.Main execution statement  610
-- UnitMgr.Main execution statement   17
-- UnitMgr.Main execution statement   29
      return Ret_Value;

   end Trans_Done;


   function State_by_Br_Is (
      UnId : Unit_ID_T;
      BrId : Branch_ID_T;
      State : Branch_State_T )
   return Boolean
   is
      Ret_Value : Boolean;
      Ret_Value_from_Call : Boolean;
   begin

-- ModeMgr.Main execution statement  117
-- ModeMgr.Main execution statement  180
-- ModeMgr.Main execution statement  283
-- ModeMgr.Main execution statement  364
-- ModeMgr.Main execution statement  398
-- ModeMgr.Main execution statement  410
-- ModeMgr.Main execution statement  436
-- ModeMgr.Main execution statement  491
-- ModeMgr.Main execution statement  503
-- ModeMgr.Main execution statement  545
-- ModeMgr.Main execution statement  606
      Ret_Value := not Units(UnId).Reconf_Ongoing;
-- ModeMgr.Main execution statement  118
-- ModeMgr.Main execution statement  181
-- ModeMgr.Main execution statement  284
-- ModeMgr.Main execution statement  365
-- ModeMgr.Main execution statement  399
-- ModeMgr.Main execution statement  411
-- ModeMgr.Main execution statement  437
-- ModeMgr.Main execution statement  492
-- ModeMgr.Main execution statement  504
-- ModeMgr.Main execution statement  546
-- ModeMgr.Main execution statement  607
      if Ret_Value then
-- ModeMgr.Main execution statement  119
-- ModeMgr.Main execution statement  182
-- ModeMgr.Main execution statement  285
-- ModeMgr.Main execution statement  366
-- ModeMgr.Main execution statement  400
-- ModeMgr.Main execution statement  412
-- ModeMgr.Main execution statement  438
-- ModeMgr.Main execution statement  493
-- ModeMgr.Main execution statement  505
-- ModeMgr.Main execution statement  547
-- ModeMgr.Main execution statement  608
         Ret_Value_from_Call := Trans_Done (UnId, BrId, State);
-- ModeMgr.Main execution statement  122
-- ModeMgr.Main execution statement  185
-- ModeMgr.Main execution statement  288
-- ModeMgr.Main execution statement  369
-- ModeMgr.Main execution statement  403
-- ModeMgr.Main execution statement  415
-- ModeMgr.Main execution statement  441
-- ModeMgr.Main execution statement  496
-- ModeMgr.Main execution statement  508
-- ModeMgr.Main execution statement  550
-- ModeMgr.Main execution statement  611
         Ret_Value := Ret_Value_from_Call;
      end if;
-- ModeMgr.Main execution statement  123
-- ModeMgr.Main execution statement  186
-- ModeMgr.Main execution statement  289
-- ModeMgr.Main execution statement  370
-- ModeMgr.Main execution statement  404
-- ModeMgr.Main execution statement  416
-- ModeMgr.Main execution statement  442
-- ModeMgr.Main execution statement  497
-- ModeMgr.Main execution statement  509
-- ModeMgr.Main execution statement  551
-- ModeMgr.Main execution statement  612
      return Ret_Value;

   end State_by_Br_Is;


   function State_by_U_Is (
      UnId : Unit_ID_T;
      State : Branch_State_T )
   return Boolean
   is
      Ret_Value : Boolean;
      Ret_Value_from_Call : Boolean;
   begin

-- ModeMgr.Main execution statement  363
-- ModeMgr.Main execution statement  397
-- ModeMgr.Main execution statement  409
-- ModeMgr.Main execution statement  435
-- ModeMgr.Main execution statement  490
-- ModeMgr.Main execution statement  502
-- ModeMgr.Main execution statement  544
      Ret_Value_from_Call := State_by_Br_Is (
         UnId, Units(UnId).CuBrId, State);
-- ModeMgr.Main execution statement  371
-- ModeMgr.Main execution statement  405
-- ModeMgr.Main execution statement  417
-- ModeMgr.Main execution statement  443
-- ModeMgr.Main execution statement  498
-- ModeMgr.Main execution statement  510
-- ModeMgr.Main execution statement  552
      Ret_Value := Ret_Value_from_Call;
-- ModeMgr.Main execution statement  372
-- ModeMgr.Main execution statement  406
-- ModeMgr.Main execution statement  418
-- ModeMgr.Main execution statement  444
-- ModeMgr.Main execution statement  499
-- ModeMgr.Main execution statement  511
-- ModeMgr.Main execution statement  553
      return Ret_Value;

   end State_by_U_Is;


   procedure Execute_Reconf_Step (UnId : Unit_ID_T)
   is
      -- Invariant (actually assertion at the start point):
      -- Units(UnId).Step /= 0
      -- or (Units(UnId).Br(A).State /= Off
      --     and Units(UnId).Br(B).State = Off
      --     and Units(UnId).Br(A).Status /= Free
      --     and Units(UnId).Br(B).Status = Reserved)
      Ret_Value_from_Call : Boolean;
   begin
      Ada.Text_IO.Put_Line ("RECONF STEP u="&UnId'Img
         & " " & Units(UnId).CuBrId'Img
         & " step=" & Units(UnId).Step'Img);

-- UnitMgr.Main execution statement    4
      case Units(UnId).Step is

         when 0 =>

-- UnitMgr.Main execution statement    5
            for BrId in Branch_ID_T loop
-- UnitMgr.Main execution statement    6
               Units(UnId).Br(BrId).Error := None;
-- UnitMgr.Main execution statement    7
               Units(UnId).Br(BrId).Status := Reserved;
-- UnitMgr.Main execution statement 1005
            end loop;

-- UnitMgr.Main execution statement    8
            Internal_Start_Trans (UnId, A, Off);
            -- IMPORTANT Start_Transition cannot be used here

-- UnitMgr.Main execution statement   14
            Units(UnId).Step := Units(UnId).Step + 1;


         when 1 =>

-- UnitMgr.Main execution statement   15
            Ret_Value_from_Call := Trans_Done (UnId, A, Off);
-- UnitMgr.Main execution statement   18
            if Ret_Value_from_Call then

-- UnitMgr.Main execution statement   19
               Units(UnId).CuBrId := B;
               --
               -- Now A branch is off, we have to select B branch
               -- here, since FDIR monitors only "the current branch",
               -- (otherwise FDIR would not recognize failure of the
               -- branch B swiching).
               --
               -- Note that the unit status (free/reserved/lock) is
               -- changed after the correct branch state is achieved
               -- in the next step.

-- UnitMgr.Main execution statement   20
               Internal_Start_Trans (UnId, B, Units(UnId).Target_State);
               -- IMPORTANT Start_Transition cannot be used here

-- UnitMgr.Main execution statement   26
               Units(UnId).Step := Units(UnId).Step + 1;

            end if;

         when 2 =>

-- UnitMgr.Main execution statement   27
            Ret_Value_from_Call := Trans_Done (UnId, B, Units(UnId).Target_State);
-- UnitMgr.Main execution statement 1029
            if Ret_Value_from_Call then

-- UnitMgr.Main execution statement   30
               Units(UnId).Br(B).Status := Units(UnId).Orig_Status;
-- UnitMgr.Main execution statement   31
               Units(UnId).Reconf_Ongoing := False;

            end if;

         when others =>

            null;

      end case;

   end Execute_Reconf_Step;

   procedure Release (UnId : Unit_ID_T) is
   begin
-- ModeMgr.Main execution statement   52
-- ModeMgr.Main execution statement   91
-- ModeMgr.Main execution statement  155
-- ModeMgr.Main execution statement  258
-- ModeMgr.Main execution statement  332
-- ModeMgr.Main execution statement  580
      for BrId in Branch_ID_T loop
-- ModeMgr.Main execution statement   53
-- ModeMgr.Main execution statement   92
-- ModeMgr.Main execution statement  156
-- ModeMgr.Main execution statement  259
-- ModeMgr.Main execution statement  333
-- ModeMgr.Main execution statement  581
         Start_Transition_Br (UnId, BrId, Off);
-- ModeMgr.Main execution statement 1052
-- ModeMgr.Main execution statement 1091
-- ModeMgr.Main execution statement 1155
-- ModeMgr.Main execution statement 1258
-- ModeMgr.Main execution statement 1332
-- ModeMgr.Main execution statement 1580
      end loop;
   end Release;

   procedure Release_All is
   begin
-- ModeMgr.Main execution statement   50
-- ModeMgr.Main execution statement   89
-- ModeMgr.Main execution statement  578
      for UnId in Unit_ID_T loop
-- ModeMgr.Main execution statement   51
-- ModeMgr.Main execution statement   90
-- ModeMgr.Main execution statement  579
         Release (UnId);
-- ModeMgr.Main execution statement 1050
-- ModeMgr.Main execution statement 1089
-- ModeMgr.Main execution statement 1578
      end loop;
   end Release_All;

   function All_Off return Boolean is
      Ret_Value : Boolean;
      Ret_Value_from_Call : Boolean;
   begin

-- ModeMgr.Main execution statement  114
-- ModeMgr.Main execution statement  603
      for UnId in Unit_ID_T loop
-- ModeMgr.Main execution statement  115
-- ModeMgr.Main execution statement  604
         for BrId in Branch_ID_T loop
-- ModeMgr.Main execution statement  116
-- ModeMgr.Main execution statement  605
            Ret_Value_from_Call := State_by_Br_Is (UnId, BrId, Off);
-- ModeMgr.Main execution statement  124
-- ModeMgr.Main execution statement  613
            if not Ret_Value_from_Call then
-- ModeMgr.Main execution statement  125
-- ModeMgr.Main execution statement  614
               Ret_Value := False;
-- ModeMgr.Main execution statement  126
-- ModeMgr.Main execution statement  615
               return Ret_Value;
            end if;
-- ModeMgr.Main execution statement 1115
-- ModeMgr.Main execution statement 1604
         end loop;
-- ModeMgr.Main execution statement 1114
-- ModeMgr.Main execution statement 1603
      end loop;

-- ModeMgr.Main execution statement  127
-- ModeMgr.Main execution statement  616
      Ret_Value := True;
-- ModeMgr.Main execution statement  128
-- ModeMgr.Main execution statement  617
      return Ret_Value;

   end All_Off;


   procedure Start_Mode_Specific_Unit_Conf (Mode : Mode_T) is
   begin
-- ModeMgr.Main execution statement  131
-- ModeMgr.Main execution statement  234
      for UnId in Unit_ID_T loop
-- ModeMgr.Main execution statement  132
-- ModeMgr.Main execution statement  235
         if First_State (Mode, UnId) /= Off then
-- ModeMgr.Main execution statement  133
-- ModeMgr.Main execution statement  236
            if Units(UnId).Br(Units(UnId).CuBrId).State
               /= First_State (Mode, UnId)
            then
-- ModeMgr.Main execution statement  134
-- ModeMgr.Main execution statement  237
               Start_Transition_Br (UnId, Units(UnId).CuBrId,
                                    First_State (Mode, UnId));
            end if;
         else
-- ModeMgr.Main execution statement  154
-- ModeMgr.Main execution statement  257
            Release (UnId);
         end if;
-- ModeMgr.Main execution statement 1131
-- ModeMgr.Main execution statement 1234
      end loop;
   end Start_Mode_Specific_Unit_Conf;


   function Unit_Conf_Done (Mode : Mode_T) return Boolean is
      Ret_Value : Boolean;
      Ret_Value_from_Call : Boolean;
   begin

-- ModeMgr.Main execution statement  178
-- ModeMgr.Main execution statement  281
      for UnId in Unit_ID_T loop

-- ModeMgr.Main execution statement  179
-- ModeMgr.Main execution statement  282
         Ret_Value_from_Call := State_by_Br_Is (
            UnId, Units(UnId).CuBrId, First_State (Mode, UnId));
-- ModeMgr.Main execution statement  187
-- ModeMgr.Main execution statement  290
         if not Ret_Value_from_Call then
-- ModeMgr.Main execution statement  188
-- ModeMgr.Main execution statement  291
            Ret_Value := False;
-- ModeMgr.Main execution statement  189
-- ModeMgr.Main execution statement  292
            return Ret_Value;
         end if;

-- ModeMgr.Main execution statement 1178
-- ModeMgr.Main execution statement 1281
      end loop;

-- ModeMgr.Main execution statement  190
-- ModeMgr.Main execution statement  293
      Ret_Value := True;
-- ModeMgr.Main execution statement  191
-- ModeMgr.Main execution statement  294
      return Ret_Value;

   end Unit_Conf_Done;


   procedure Lock (UnId : Unit_ID_T) is
   begin

-- ModeMgr.Main execution statement  221
-- ModeMgr.Main execution statement  324
-- ModeMgr.Main execution statement  514
      Units(UnId).Br(Units(UnId).CuBrId).Status := Locked;

   end Lock;


   procedure Lock_All_Selected (Mode : Mode_T) is
   begin

-- ModeMgr.Main execution statement  218
-- ModeMgr.Main execution statement  321
      for UnId in Unit_ID_T loop

-- ModeMgr.Main execution statement  219
-- ModeMgr.Main execution statement  322
         if First_State (Mode, UnId) /= Off then
            -- Invariant: Units(UnId).Br(Units(UnId).CuBrId).Status = Reserved

-- ModeMgr.Main execution statement  220
-- ModeMgr.Main execution statement  323
            Lock (UnId);

         end if;

-- ModeMgr.Main execution statement 1218
-- ModeMgr.Main execution statement 1321
      end loop;

   end Lock_All_Selected;

end UnitMgr;
